/*
 *  Copyright (C) 2005 Mohamed Louizi
 *  Copyright (C) 2006,2007 Christoph Sommer <christoph.sommer@informatik.uni-erlangen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "networklayer/dtrb/DTRB_RouteEntry.h"
#include "networklayer/dtrb/DTRB.h"

namespace DTRB {

DTRB_RouteEntry::DTRB_RouteEntry(DTRB_Net* dtrb) : routingEntry(0), dtrb(dtrb) {

}

DTRB_RouteEntry::~DTRB_RouteEntry(){

}

std::string DTRB_RouteEntry::info() const {
	std::ostringstream ss;

	ss << qValues.size() << " entries";

	ss << " {" << std::endl;
	for (unsigned int i = 0; i < qValues.size(); i++)
	    {
		ss << "BythewayNode:";
		ss << "  " << qValues[i].byTheWayNode << std::endl;
		ss << ">";
		ss << "  " << qValues[i].qValue << std::endl;
	}
	ss << "}";

	return ss.str();
}

std::string DTRB_RouteEntry::detailedInfo() const {
	return info();
}


void DTRB_RouteEntry::setQvalue(IPAddress bwNode, double reward, double delta){

    for (unsigned int i = 0; i < qValues.size(); i++)
    {
        if (qValues[i].byTheWayNode == bwNode)
        {
        	qValues[i].qValue = (qValues[i].qValue*(1-delta))+(reward*delta);
            return;
        }
    }
    double newReward = 0;
    newReward = (newReward*(1-delta))+(reward*delta);
    Q newQvalue;
    newQvalue.byTheWayNode = bwNode;
    newQvalue.qValue = newReward;
	qValues.push_back(newQvalue);
}

double DTRB_RouteEntry::maxQvalue(){

	double qV = 0;
	double R = 0;
	simtime_t now = simTime();
	if ((now - this->msgTime + this->distTime) <= 5.0)
		R = 1;
	if (((now - this->msgTime + this->distTime) > 5.0) && ((now - this->msgTime + this->distTime) <= 10.0))
		R = 0.75;
	if (((now - this->msgTime + this->distTime) > 10.0) && ((now - this->msgTime + this->distTime) <= 15.0))
		R = 0.50;
	if (((now - this->msgTime + this->distTime) > 15.0) && ((now - this->msgTime + this->distTime) <= 20.0))
		R = 0.25;

	for (unsigned int i = 0; i < qValues.size(); i++)
	    {
			if (qValues[i].qValue > qV)
				qV = qValues[i].qValue; //MF discount factor...
	    }
	return (qV + R);

}

void DTRB_RouteEntry::ageDeliveryPreds(double mult){

	for (unsigned int i = 0; i < qValues.size(); i++)
		{
		if (qValues[i].qValue > 0)
			qValues[i].qValue = (qValues[i].qValue)*mult;
		}

}


std::ostream& operator<<(std::ostream& os, const DTRB_RouteEntry& o)
{

	os << "[ ";
	os << "nodeAddress: " << o.nodeAddress;

	cModule* router = o.dtrb->getRouterByAddress(o.nodeAddress);
	os << " (" << (router ? router->getFullName() : "?") << ")";

	os << ", ";
	os << "distTime: " << o.distTime;
	os << ", ";
	os << "msgTime: " << o.msgTime;
	os << ", ";
	os << "Qvalues: ";
	os << o.info();
	os << " ]";

	return os;
};

}
