/*
 *  Copyright (C) 2005 Mohamed Louizi
 *  Copyright (C) 2006,2007 Christoph Sommer <christoph.sommer@informatik.uni-erlangen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef NETWORKLAYER_DTRB_DTRB_ROUTEENTRY_H
#define NETWORKLAYER_DTRB_DTRB_ROUTEENTRY_H

#include <omnetpp.h>
#include <string.h>
#include <sstream>

#include <IPAddress.h>
#include <InterfaceTable.h>
#include "RoutingTable.h"
//#include "DTRB_Timer.h"

class DTRB_Net;

namespace DTRB {
/**
 * DTRB Route Table Entry
 */
class DTRB_RouteEntry
{
  public:
    DTRB_RouteEntry(DTRB_Net* dtrb);
    virtual ~DTRB_RouteEntry();

    IPAddress nodeAddress;
    simtime_t msgTime;
    simtime_t distTime;

    void setQvalue(IPAddress bwNode, double reward, double delta);
    double maxQvalue();
    void ageDeliveryPreds(double mult);

    /** @brief inherited from cObject */
   virtual std::string info() const;

   /** @brief inherited from cObject */
   virtual std::string detailedInfo() const;


    //int nodePrefix;

    IPRoute* routingEntry; /**< Forwarding Route (entry in standard OMNeT++ routingTable) */

  private:
    typedef struct Q{
    	IPAddress byTheWayNode;
		double qValue;
		//Q(IPAddress bw, int r) : byTheWayNode(bw), qValue(r){}
	} Q ;
	typedef std::vector< Q > QValues;
	QValues qValues;

  protected:
    DTRB_Net* dtrb; /**< DTRB module */

  public:
    friend std::ostream& operator<<(std::ostream& os, const DTRB_RouteEntry& e);
};
}

#endif
