/*
 *  Copyright (C) 2005 Mohamed Louizi
 *  Copyright (C) 2006,2007 Christoph Sommer <christoph.sommer@informatik.uni-erlangen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdexcept>
#include <sstream>
#include <algorithm>
#include "networklayer/dtrb/DTRB_RouteTable.h"
#include "IPAddressResolver.h"
#include "IPv4InterfaceData.h"


namespace DTRB {

DTRB_RouteTable::DTRB_RouteTable(cModule* , const IPAddress& , const char* , const IPAddress& ) {

}

DTRB_RouteTable::~DTRB_RouteTable() {
	DTRB_RouteEntry* entry;
	while ((entry = getRoute(0))) deleteRoute(entry);
}

const char* DTRB_RouteTable::getFullName() const {
	return "DTRB_RouteTable";
}

std::string DTRB_RouteTable::info() const {
	std::ostringstream ss;

	ss << getNumRoutes() << " entries";

	ss << " {" << std::endl;
	for (std::vector<DTRB_RouteEntry *>::const_iterator iter = routeVector.begin(); iter < routeVector.end(); iter++) {
		DTRB_RouteEntry* e = *iter;
		ss << "  " << *e << std::endl;
	}
	ss << "}";

	return ss.str();
}

std::string DTRB_RouteTable::detailedInfo() const {
	return info();
}

//=================================================================================================
/*
 * Function returns the size of the table
 */
//=================================================================================================
int DTRB_RouteTable::getNumRoutes() const {
  return (int)routeVector.size();
}

//=================================================================================================
/*
 * Function gets an route entry at the given position
 */
//=================================================================================================
DTRB_RouteEntry* DTRB_RouteTable::getRoute(int k){
  if(k < (int)routeVector.size())
    return routeVector[k];
  else
    return NULL;
}

//=================================================================================================
/*
 *
 */
//=================================================================================================
void DTRB_RouteTable::addRoute(DTRB_RouteEntry *entry){
	routeVector.push_back(entry);
}

//=================================================================================================
/*
 */
//=================================================================================================
void DTRB_RouteTable::deleteRoute(DTRB_RouteEntry *entry){

	// update DTRB routeTable
	std::vector<DTRB_RouteEntry *>::iterator iter;
	for(iter = routeVector.begin(); iter < routeVector.end(); iter++){
		if(entry == *iter){
			routeVector.erase(iter);
			//updateDisplayString();
			delete entry;
			return;
		}
	}

	throw std::runtime_error("unknown route entry requested to be deleted");
}


//=================================================================================================
/*
 */
//=================================================================================================
DTRB_RouteEntry* DTRB_RouteTable::getByNode(IPAddress addr)
{

    std::vector<DTRB_RouteEntry *>::iterator iter;

    for (iter = routeVector.begin(); iter < routeVector.end(); iter++)
    {
        DTRB_RouteEntry *entry = *iter;

        if (entry->nodeAddress == addr)
        {
            return entry;
        }
    }

    return 0;
}





//=================================================================================================
/*
 */
//=================================================================================================
DTRB_RouteTable::RouteVector DTRB_RouteTable::getRouteTable(){
	return routeVector;
}



std::ostream& operator<<(std::ostream& os, const DTRB_RouteTable& o)
{
	os << o.info();
	return os;
}
}
