/*
 *  Copyright (C) 2005 Mohamed Louizi
 *  Copyright (C) 2006,2007 Christoph Sommer <christoph.sommer@informatik.uni-erlangen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DTRB_ROUTETABLE_H
#define DTRB_ROUTETABLE_H

#include <omnetpp.h>
#include <vector>
#include "INETDefs.h"

#include "NotificationBoard.h"

#include "DTRB_RouteEntry.h"

namespace DTRB {
/**
  * class describes the functionality of the route table
**/
class DTRB_RouteTable : public cObject
{
  public:
	DTRB_RouteTable(cModule* host, const IPAddress& myAddr, const char* DTRB_INTERFACES, const IPAddress& LL_MANET_ROUTERS);
	virtual ~DTRB_RouteTable();

    /** @brief inherited from cObject */
    virtual const char* getFullName() const;

    /** @brief inherited from cObject */
    virtual std::string info() const;

    /** @brief inherited from cObject */
    virtual std::string detailedInfo() const;

    //-----------------------------------------------------------------------
    //Route table manupilation operations
    //-----------------------------------------------------------------------
    /** @returns the size of the table **/
    int getNumRoutes() const;
    /** @gets an route entry at the given position **/
    DTRB_RouteEntry* getRoute(int k);
    /** @adds a new entry to the table **/
    void addRoute(DTRB_RouteEntry *entry);
    /** @deletes an entry from the table **/
    void deleteRoute (DTRB_RouteEntry *entry);
    /** @removes invalid routes from the network layer route table **/
    //void maintainAssociatedRouteTable ();
    /** @searchs an entry (exact match) and gives back a pointer to it, or 0 if none is found **/
    DTRB_RouteEntry* getByNode(IPAddress addr);
    //DTRB_RouteEntry* getByDest(IPAddress addr);
    /** @searchs an entry (longest-prefix match) and gives back a pointer to it, or 0 if none is found **/
    //DTRB_RouteEntry* getForAddress(IPAddress addr);
    /** @returns the route table **/
    std::vector<DTRB_RouteEntry *> getRouteTable();

  private:
    typedef std::vector<DTRB_RouteEntry *> RouteVector;
	RouteVector routeVector;

	NotificationBoard *nb;

	//IRoutingTable* routeTable; /**< containing host's standard IRouteTable module */

	/**
	 * add or delete network layer route table entry for given DTRB route table entry, based on whether it's valid
	 */
	//void maintainAssociatedRouteEntryFor(DTRB_RouteEntry* entry);

	public:
		friend std::ostream& operator<<(std::ostream& os, const DTRB_RouteTable& o);

    };
}

typedef DTRB::DTRB_RouteTable DTRB_Net_RouteTable;

#endif
